from django.db.models import Lookup, lookups


class CompareCITextLookupMixin:
    def process_lhs(self, compiler, connection, lhs=None):
        lhs_sql, params = Lookup.process_lhs(self, compiler, connection, lhs)
        return "%s::citext" % lhs_sql, list(params)


class InsensitiveCompareCITextLookupMixin:
    def process_lhs(self, compiler, connection, lhs=None):
        lhs_sql, params = Lookup.process_lhs(self, compiler, connection, lhs)
        return "UPPER(%s)" % lhs_sql, list(params)


class IsNull(lookups.IsNull):
    def process_lhs(self, compiler, connection, lhs=None):
        lhs_sql, params = Lookup.process_lhs(self, compiler, connection, lhs)
        return "%s::text" % lhs_sql, list(params)


class Contains(CompareCITextLookupMixin, lookups.Contains):
    pass


class StartsWith(CompareCITextLookupMixin, lookups.StartsWith):
    pass


class EndsWith(CompareCITextLookupMixin, lookups.EndsWith):
    pass


class Regex(CompareCITextLookupMixin, lookups.Regex):
    pass


class IRegex(CompareCITextLookupMixin, lookups.IRegex):
    pass


class IContains(InsensitiveCompareCITextLookupMixin, lookups.IContains):
    pass


class IStartsWith(InsensitiveCompareCITextLookupMixin, lookups.IStartsWith):
    pass


class IEndsWith(InsensitiveCompareCITextLookupMixin, lookups.IEndsWith):
    pass


class IExact(InsensitiveCompareCITextLookupMixin, lookups.IExact):
    pass



"""
Extends model_bakery's generators to support citext fields.

This logic is loaded only if package model_bakery is installed.
https://github.com/model-bakers/model_bakery/
"""

try:
    from model_bakery import baker, random_gen
except ImportError:
    pass
else:
    baker.generators.add("citext.fields.CICharField", random_gen.gen_string)
    baker.generators.add("citext.fields.CIEmailField", random_gen.gen_email)
    baker.generators.add("citext.fields.CITextField", random_gen.gen_text)



from django.db import models

from . import lookups

__all__ = ["CICharField", "CIEmailField", "CIText", "CITextField"]


class CIText:
    def get_internal_type(self):
        return "CI" + super().get_internal_type()

    def db_type(self, connection):
        return "citext"


class CICharField(CIText, models.CharField):
    pass


class CIEmailField(CIText, models.EmailField):
    pass


class CITextField(CIText, models.TextField):
    pass


field: models.Field

for field in CIText.__subclasses__():
    for lookup in lookups.CompareCITextLookupMixin.__subclasses__():
        field.register_lookup(lookup)

    for lookup in lookups.InsensitiveCompareCITextLookupMixin.__subclasses__():
        field.register_lookup(lookup)

    field.register_lookup(lookups.IsNull)


from . import baker_generators  # noqa
